
local AceOO = AceLibrary( "AceOO-2.0" )

	
function ARKCORE_Table_Clean( d )

	-- nil all values in a table including subtables, so they can be garbage collected
	
	if type( d ) ~= "table" then
		return
	end
		
	for k in pairs( d ) do
	
		if type( d[k] ) == "table" then
			-- recurse if a table has another table as an element
			ARKCORE_Table_Clean( d[k] )
		end
		
		-- clear value
		d[k] = nil
	
	end

	d = nil

end



ARKCORE_Class_uiFrame = AceOO.Class()

function ARKCORE_Class_uiFrame.prototype:init( frameName, frameType, frameParent, frameInherit )

	ARKCORE_Class_uiFrame.super.prototype.init( self )

	self.widget = getglobal( frameName )
	
	if not self.widget then
		if frameType == false then
			error( "XML Frame " .. frameName .. " does not exist" )
		else
			self.widget = CreateFrame( frameType, frameName, frameParent, frameInherit )
		end
	end
	
end

function ARKCORE_Class_uiFrame.prototype:tostring()
	if self.widget then
		return self.widget:GetName()
	else
		return "nil"
	end
end









ARKCORE_Class_StealthTooltip = AceOO.Class( ARKCORE_Class_uiFrame )

function ARKCORE_Class_StealthTooltip.prototype:init( frame )

	ARKCORE_Class_StealthTooltip.super.prototype.init( self, frame, false )

	assert( self.widget, "Tooltip frame " .. tostring( frame ) .. " not found" )
	
end

function ARKCORE_Class_StealthTooltip.prototype:tostring()
	if self.widget then
		return self.widget:GetName()
	else
		return "nil"
	end
end

function ARKCORE_Class_StealthTooltip.prototype:SetHyperlink( hyperlink )

	self.widget:ClearLines()
	
	if hyperlink then
		self.widget:SetHyperlink( hyperlink )
	end
	
end

function ARKCORE_Class_StealthTooltip.prototype:SetBagItem( bagnum, slotnum )

	self.widget:ClearLines()

	if bagnum and slotnum then
	
		if bagnum == BANK_CONTAINER then
			self.widget:SetInventoryItem( "player", BankButtonIDToInvSlotID( slotnum ) )
		elseif bagnum == KEYRING_CONTAINER then
			self.widget:SetInventoryItem( "player", KeyRingButtonIDToInvSlotID( slotnum ) )
		else
			self.widget:SetBagItem( bagnum, slotnum )
		end
		
	end
	
end

function ARKCORE_Class_StealthTooltip.prototype:Find( TextToFind, IgnoreLeft, IgnoreRight, CaseSensitive )

	if not TextToFind or strtrim( TextToFind ) == "" then
		return false
	end
	
	local IgnoreLeft = IgnoreLeft or false
	local IgnoreRight = IgnoreRight or false
	local CaseSensitive = CaseSensitive or false
	
	local obj, txt
	
	for i = 2, self.widget:NumLines() do
	
		if not IgnoreLeft then
			obj = getglobal( self.widget:GetName() .. "TextLeft" .. i )
			if obj then
				txt = obj:GetText()
				if txt then
					if not CaseSensitive then
						txt = string.lower( txt )
						TextToFind = string.lower( TextToFind )
					end
					if string.find( txt, TextToFind ) then
						return string.find( txt, TextToFind )
					end
				end
			end
		end
		
		if not IgnoreRight then
			obj = getglobal( self.widget:GetName() .. "TextRight" .. i )
			if obj then
				txt = obj:GetText()
				if txt then
					if not CaseSensitive then
						txt = string.lower( txt )
						TextToFind = string.lower( TextToFind )
					end
					if string.find( txt, TextToFind ) then
						return string.find( txt, TextToFind )
					end
				end
			end
		end

	end

	return nil
	
end

function ARKCORE_Class_StealthTooltip.prototype:Contains( TextToFind, IgnoreLeft, IgnoreRight, CaseSensitive )

	if self:Find( TextToFind, IgnoreLeft, IgnoreRight, CaseSensitive ) then
		return true
	else
		return false
	end

end








--[[

local uiSlot = AceOO.Class( ARKCORE_Class_uiFrame )

function uiSlot.prototype:new( parent, slotnum )

	local parentname = parent.frame:GetName()
	local framename = parentname .. "Slot" .. slotnum
	
	local frame = getglobal( framename )
	if not frame then
		frame = CreateFrame( "Button", framename, parent.frame, "ARKINV_TemplateButtonItem" )
	end
	frame:SetID( slotnum )
	frame:Hide()
	
	self.xml_Frame = frame

	self.parent = parent
	self.slotnum = slotnum
	
	self.xml_ArkQualityBorder = getglobal( framename .. "ArkQualityBorder" )
	self.xml_ArkHighlight = getglobal( framename .. "ArkHighlight" )
	self.xml_ArkNew = getglobal( framename .. "ArkNewText" )
	
	self.xml_Cooldown = getglobal( framename .. "Cooldown" )
	
end










ARKCORE_Class_uiContainer = AceOO.Class()

function ARKCORE_Class_uiContainer.prototype:new( Id )

	local parent = ARKINV_G.GUI_NAME .. Id
	local name = parent .. "Container"
	
	self.frame = ARKCORE_Class_uiFrame:new( name, "Frame", parent, inherits )
	self.frame:SetID( Id )
	
	self.Bag = ARKCORE_Class_Table:new()
	for x = -2, 11 do
		self.Bag[x] = ARKCORE_Class_Bag:new()
	end
	
	self.Bar = ARKCORE_Class_Table:new()
	self.BarCount = 0
	self.BarMax = 0

end











local uiBag = AceOO.Class( ARKCORE_Class_uiFrame )

uiBag.prototype.frame = false
uiBag.prototype.bagnum = false
uiBag.prototype.slot = { }

function uiBag.prototype:new( bagnum )

	local loc_id = ARKINV:BagLocation( id )

	local parentname = ARKINV_G.GUI_NAME .. loc_id .. "Container" .. "Bag"
	local framename = parentname .. bagnum

	local frame = getglobal( framename )
	if not frame then
		frame = CreateFrame( "Frame", framename, getglobal( parentname ) )
	end
	
	frame:SetWidth( 1 )
	frame:SetHeight( 1 )
	frame:SetPoint( "TOPLEFT", 0, 0 )
	frame:SetID( bagnum )
	frame:Show()
	
	self.xml_Frame = frame
	self.loc_id = loc_id
	self.bagnum = bagnum
	
end



function uiSlot.prototype:Show()

	if self.xml_Frame then

		if ARKINV_G.MODE.EDIT or ARKINV_G.LOCATION[self.parent.loc_id].offline then
			-- disable clicks/drag when in edit mode or offline
			self.xml_Frame:RegisterForClicks()
			self.xml_Frame:RegisterForDrag()
		else
			self.xml_Frame:RegisterForClicks( "LeftButtonUp", "RightButtonUp" )
			self.xml_Frame:RegisterForDrag( "LeftButton" )
		end

		i = self.db.account.cache.who[current_player].loc[self.parent.loc_id].bag[self.parent.bagnum].slot[slotnum]
		
		if ARKINV_G.LOCATION[loc_id].rebuild then

			if not self.linked  then
			
				-- anchor unlinked item to bar frame
				self.xml_Frame:SetPoint( "BOTTOMRIGHT", self.bar, "BOTTOMRIGHT", 0 - pad_bar, pad_bar )
				
			else
			
				if ( mod( ( j - 1 ), col ) == 0 ) then
					-- next row, anchor to first item in previous row
					self.xml_Frame:SetPoint( "BOTTOMRIGHT", self.linked, "TOPRIGHT", 0, pad_slot )
				else
					-- anchor to last one
					self.xml_Frame:SetPoint( "BOTTOMRIGHT", self.linked, "BOTTOMLEFT", 0 - pad_slot, 0 )
				end
			end
	
			SetItemButtonTexture( self.xml_Frame, ARKINV:GetItemTexture( i.h ) )
		
		end

		SetItemButtonCount( self.xml_Frame, i.count )
	
		-- item locked, needs to only happen for online items
		if not ARKINV_G.LOCATION[loc_id].offline then
			local _, _, locked, _, readable = GetContainerItemInfo( self.bagnum, self.slotnum )
			self.xml_Frame.readable = readable
			SetItemButtonDesaturated( self.xml_Frame, locked, 0.5, 0.5, 0.5 )
		else
			self.xml_Frame.readable = false
		end

		if ARKINV_G.LOCATION[loc_id].offline and self.db.account.option.offline.fade then
			self.xml_Frame:SetAlpha( ARKINV_G.FADE_ALPHA )
		else
			self.xml_Frame:SetAlpha( 1 )
		end
		
		-- item rarity coloured borders
		if i.q > 0 and ARKINV:OptionLocalGet( loc_id, { "slot", "border" } ) then
			local r, g, b = GetItemQualityColor( i.q )
			self.xml_QualityBorder:SetBackdropBorderColor( r, g, b, 1 )
		else
			self.xml_QualityBorder:SetBackdropBorderColor( 0.5, 0.5, 0.5, 1 )
		end


		if i.h then
			
			ContainerFrame_UpdateCooldown( self.parent.bagnum, self.xml_Frame )
			self.xml_Frame.hasItem = 1

		else
			
			-- empty slot colours
			local bt = ARKINV.db.account.cache.who[current_player].loc[loc_id].bag[self.parent.bagnum].type
			local colour = ARKINV:OptionLocalGet( loc_id, { "slot", "data", bt, "colour" } )

			-- icon or coloured background
			if ARKINV:OptionLocalGet( loc_id, { "slot", "empty", "icon" } ) then
				SetItemButtonTexture( self.xml_Frame, "Interface\\PaperDoll\\UI-Backpack-EmptySlot" )
			else
				SetItemButtonTexture( self.xml_Frame, "Interface\\Buttons\\WHITE8X8" )
				SetItemButtonTextureVertexColor( self.xml_Frame, colour.r * 0.25, colour.g * 0.25, colour.b * 0.25 )
			end

			-- empty borders
			if ARKINV:OptionLocalGet( loc_id, { "slot", "empty", "border" } ) then
				self.xml_ArkQualityBorder:SetBackdropBorderColor( colour.r, colour.g, colour.b, 1 )
			else
				self.xml_ArkQualityBorder:SetBackdropBorderColor( 0.5, 0.5, 0.5, 1 )
			end

			self.xml_Cooldown:Hide()
			obj.hasItem = nil
			
		end
		
		self.xml_Frame:Show()
		
	end

end



local uiBar = AceOO.Class( ARKCORE_Class_uiFrame )

uiBar.prototype.xml_Frame = false
uiBar.prototype.xml_Background = false
uiBar.prototype.xml_Edit = false

uiBar.prototype.loc_id = false
uiBar.prototype.barnum = false
uiBar.prototype.linked = false
uiBar.prototype.width = 0
uiBar.prototype.height = 0
uiBar.prototype.slot = { }


function uiBar.prototype:new( loc_id, barnum )

	local parentname = ARKINV_G.GUI_NAME .. loc_id .. "Container" .. "Bar"
	local framename = parentname .. barnum

	local frame = getglobal( framename )
	if not frame then
		frame = CreateFrame( "Frame", framename, getglobal( parentname ), "ARKINV_TemplateFrameBar" )
	end
	
	frame:SetWidth( 1 )
	frame:SetHeight( 1 )
	frame:SetPoint( "TOPLEFT", 0, 0 )
	frame:SetID( bagnum )
	frame:Show()
	
	self.xml_Frame = frame
	self.xml_Background = getglobal( framename .. "Background" )
	self.xml_Edit = getglobal( framename .. "Edit" )

	self.loc_id = loc_id
	self.barnum = barnum
	
end

function uiBar.prototype:Show()

	if self.xml_Frame then
	
		local loc_id = self.loc_id

		if ARKINV:OptionLocalGet( loc_id, { "bar", "border" } ) then
		
			local colour = ARKINV:OptionLocalGet( loc_id, { "bar", "colour", "border" } )
			self.xml_Frame:SetBackdropBorderColor( colour.r, colour.g, colour.b, 1 )
			
		else
		
			self.xml_Frame:SetBackdropBorderColor( 0, 0, 0, 0 )
			
		end

		
		if ARKINV_G.MODE.EDIT then
		
			self.xml_Frame:SetBackdropBorderColor( 1, 0, 0, 1 )
			self.xml_Background:SetTexture( 1, 0, 0, 0.1 )
			self.xml_Edit:Show()
			self.xml_Frame:Show()
			
		else
		
			local colour = ARKINV:OptionLocalGet( loc_id, { "bar", "colour", "background" } )
			self.xml_Background:SetTexture( colour.r, colour.g, colour.b, colour.a )

			-- hide the bar frames if the option is enabled (for a bug with bars coming over the top of the items)
			if not ARKINV:OptionLocalGet( loc_id, { "bar", "hide" } ) then
				self.xml_Frame:Show()
			end
			
		end

		
		-- SHOW ITEMS
		
	end

end

]]--
